import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.StringReader;

import java.text.DateFormat;
import java.text.SimpleDateFormat;

import java.util.Date;

import java.util.concurrent.LinkedBlockingQueue;

import org.xml.sax.InputSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

class data {
  public static final int num_elements = 20;
  public String[] contents = new String[num_elements];
  public String[] name     = new String[num_elements];
}

class queue {
  public LinkedBlockingQueue<data>   q   = new LinkedBlockingQueue<data>();
  public LinkedBlockingQueue<String> out = new LinkedBlockingQueue<String>();
}

class output implements Runnable {
  private queue q;

  public output(queue q) {
    this.q    = q;
  }

  public void run() {
    while (true) {
      try {

        String line = q.out.take();

        if (line.equals("*** QUIT ***")) {
          return;
        }

        System.out.println(line);

      } catch (Exception ex) {
        ex.printStackTrace();
      }
    }
  }
}

class producer implements Runnable {

  private data  d;
  private int   index;
  private queue q;
  private File  root;

  public producer(queue q, File root) {
    this.q     = q;
    this.root  = root;
    this.d     = new data();
    this.index = 0;
  }

  private void process_item(File item) {
    try {
      BufferedReader r    = new BufferedReader(new FileReader(item));
      String         line = r.readLine();
      String         name = item.getName();

      while (line != null) {
        d.contents[index] = line;
        d.name[index] = name;
        index += 1;
        line = r.readLine();
      }
      r.close();
      if (index == d.num_elements) {
        q.q.add(d);
        d = new data();
        index = 0;
      }
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  private void process_dir(File dir) {
    String[] items = dir.list();
    for (String item : items) {
      File f = new File(dir, item);
      if (f.isDirectory()) {
        process_dir(f);
      } else {
        if (item.endsWith(".atom")) {
          process_item(f);
        }
      }
    }
  }

  public void run() {
    process_dir(root);
    // add any unprocessed entries to the queue
    q.q.add(d);
    for (int i = 0; i < 6; i++) {
      d = new data();
      d.name[0] = "*** QUIT ***";
      q.q.add(d);
    }
  }

}

class consumer implements Runnable {

  private queue q;
  private DocumentBuilder db; 

  public consumer(queue q) {
    this.q    = q;
    try {
      db = DocumentBuilderFactory.newInstance().newDocumentBuilder();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  public void run() {
    while (true) {
      String name = "<ignore>";
      try {

        data   d    = q.q.take();

        for (int i = 0; i < d.num_elements; i++) {

          String line = d.contents[i];
          name        = d.name[i];

          if (name.equals("*** QUIT ***")) {
            return;
          }

          int count = 0;

          int pos = line.indexOf("</feed>");
          while (pos != -1) {
            String   feed = line.substring(0, pos+7);
            Document doc  = db.parse(new InputSource(new StringReader(feed)));

            NodeList entries = doc.getElementsByTagName("entry");

            count += entries.getLength();

            line = line.substring(pos+7, line.length());
            pos  = line.indexOf("</feed>");
          }
          q.out.add(name + ": " + count + " entries.");
        }
      } catch (Exception ex) {
        ex.printStackTrace();
        q.out.add(name + ": " + "***ERROR***");
      }
    }
  }
}

public class extract_info {

    //private static DateFormat dfp =
      //new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss+00:00");
    //private static DateFormat df = new SimpleDateFormat();

    //NodeList entries = doc.getElementsByTagName("entry");
    //for (int i = 0; i < entries.getLength(); i++) {
      //Element entry     = (Element)entries.item(i);
      //NodeList kids     = entry.getElementsByTagName("published");
      //Element published = (Element)kids.item(0);
      //Date timestamp = dfp.parse(published.getTextContent());
      //System.out.println(df.format(timestamp));
    //}

  public static void main(String[] args) {
    try {

      queue q = new queue();

      Thread o = new Thread(new output(q));
      o.start();
    
      String dir  = System.getProperty("user.dir");
      File   root = new File(dir);

      Thread p = new Thread(new producer(q, root));
      p.start();

      // give producer thread a 15 second head start
      Thread.sleep(15000);

      Thread[] consumers = new Thread[6];

      for (int i = 0; i < 6; i++) {
        consumers[i] = new Thread(new consumer(q));
        consumers[i].start();
      }

      p.join();

      for (int i = 0; i < 6; i++) {
        consumers[i].join();
      }

      q.out.add("*** QUIT ***");

      o.join();

    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }
}
